@@ -1,3 +1,9 @@
+"""
+Forum models:
+
+Category, Forum, Topic, Post, Profile, Attachment, ReadTracking
+
+"""
 from datetime import datetime
 from markdown import Markdown
 import os.path
@@ -14,11 +20,11 @@ from django.utils.html import strip_tags
 from django.utils.translation import ugettext_lazy as _
 from django.conf import settings
 
-from common.fields import AutoOneToOneField, JSONField
-
 from pybb.markups import mypostmarkup
 from pybb.util import urlize, unescape
 
+from common.fields import JSONField
+
 
 TZ_CHOICES = [(float(x[0]), x[1]) for x in (
     (-12, '-12'), (-11, '-11'), (-10, '-10'), (-9.5, '-09.5'), (-9, '-09'),
@@ -39,6 +45,7 @@ MARKUP_CHOICES = (
 class Category(models.Model):
     name = models.CharField(_('Name'), max_length=80)
     position = models.IntegerField(_('Position'), blank=True, default=0)
+    slug = models.CharField(max_length=30, db_index=True, blank=True)
 
     class Meta:
         ordering = ['position']
@@ -52,7 +59,7 @@ class Category(models.Model):
         return self.forums.all().count()
 
     def get_absolute_url(self):
-        return reverse('pybb_category', args=[self.id])
+        return reverse('pybb_category_details', args=[self.id])
 
     @property
     def topics(self):
@@ -72,6 +79,8 @@ class Forum(models.Model):
     updated = models.DateTimeField(_('Updated'), blank=True, null=True)
     post_count = models.IntegerField(_('Post count'), blank=True, default=0)
     topic_count = models.IntegerField(_('Topic count'), blank=True, default=0)
+    last_post = models.ForeignKey("Post", related_name='last_post_in_forum', verbose_name=_(u"last post"), blank=True, null=True)
+    slug = models.CharField(max_length=30, db_index=True, blank=True)
 
     class Meta:
         ordering = ['position']
@@ -83,21 +92,19 @@ class Forum(models.Model):
 
     def update_post_count(self):
         self.post_count = Topic.objects.filter(forum=self).aggregate(
-                                        Sum("post_count"))['post_count__sum']
+                                Sum("post_count"))['post_count__sum'] or 0
         self.save()
 
     def get_absolute_url(self):
-        return reverse('pybb_forum', args=[self.id])
+        return reverse('pybb_forum_details', args=[self.id])
 
     @property
     def posts(self):
         return Post.objects.filter(topic__forum=self).select_related()
 
-    @property
-    def last_post(self):
-        posts = self.posts.order_by('-created').select_related()
+    def get_last_post(self):
         try:
-            return posts[0]
+            return self.posts.order_by('-created').select_related()[0]
         except IndexError:
             return None
 
@@ -107,12 +114,13 @@ class Topic(models.Model):
     name = models.CharField(_('Subject'), max_length=255)
     created = models.DateTimeField(_('Created'), null=True)
     updated = models.DateTimeField(_('Updated'), null=True)
-    user = models.ForeignKey(User, verbose_name=_('User'))
+    user = models.ForeignKey(User, verbose_name=_('User'), related_name='pybb_topics')
     views = models.IntegerField(_('Views count'), blank=True, default=0)
     sticky = models.BooleanField(_('Sticky'), blank=True, default=False)
     closed = models.BooleanField(_('Closed'), blank=True, default=False)
     subscribers = models.ManyToManyField(User, related_name='subscriptions', verbose_name=_('Subscribers'), blank=True)
     post_count = models.IntegerField(_('Post count'), blank=True, default=0)
+    last_post = models.ForeignKey("Post", related_name="last_post_in_topic", verbose_name=_(u"last post"), blank=True, null=True)
 
     class Meta:
         ordering = ['-created']
@@ -124,14 +132,15 @@ class Topic(models.Model):
 
     @property
     def head(self):
-        return self.posts.all().order_by('created').select_related()[0]
+        if not hasattr(self, "_head"):
+            self._head = self.posts.all().order_by('created')[0]
+        return self._head
 
-    @property
-    def last_post(self):
-        return self.posts.all().order_by('-created').select_related()[0]
+    def get_last_post(self):
+        return self.posts.order_by('-created').select_related()[0]
 
     def get_absolute_url(self):
-        return reverse('pybb_topic', args=[self.id])
+        return reverse('pybb_topic_details', args=[self.id])
 
     def save(self, *args, **kwargs):
         if self.id is None:
@@ -171,7 +180,7 @@ class RenderableItem(models.Model):
 
 class Post(RenderableItem):
     topic = models.ForeignKey(Topic, related_name='posts', verbose_name=_('Topic'))
-    user = models.ForeignKey(User, related_name='posts', verbose_name=_('User'))
+    user = models.ForeignKey(User, related_name='pybb_posts', verbose_name=_('User'))
     created = models.DateTimeField(_('Created'), blank=True)
     updated = models.DateTimeField(_('Updated'), blank=True, null=True)
     markup = models.CharField(_('Markup'), max_length=15, default=settings.PYBB_DEFAULT_MARKUP, choices=MARKUP_CHOICES)
@@ -180,7 +189,6 @@ class Post(RenderableItem):
     body_text = models.TextField(_('Text version'))
     user_ip = models.IPAddressField(_('User IP'), blank=True, default='0.0.0.0')
 
-
     class Meta:
         ordering = ['created']
         verbose_name = _('Post')
@@ -205,23 +213,32 @@ class Post(RenderableItem):
 
         if new:
             self.topic.updated = now
+            self.topic.last_post = self
             self.topic.update_post_count()
             self.topic.forum.updated = now
+            self.topic.forum.last_post = self
             self.topic.forum.update_post_count()
 
-
     def get_absolute_url(self):
-        return reverse('pybb_post', args=[self.id])
-
+        return reverse('pybb_post_details', args=[self.id])
 
     def delete(self, *args, **kwargs):
         self_id = self.id
         head_post_id = self.topic.posts.order_by('created')[0].id
-        super(Post, self).delete(*args, **kwargs)
+        last_posts = list(self.topic.posts.order_by('-created')[:2])
+        last_post_id = self.topic.get_last_post().id
 
         if self_id == head_post_id:
             self.topic.delete()
+        elif self_id == last_post_id:
+            self.topic.last_post = last_posts[-1]
+            self.topic.save()
+            self.topic.forum.last_post = self.topic.last_post
+            self.topic.forum.save()
+            super(Post, self).delete(*args, **kwargs)
+            self.topic.forum.last_post = self.topic.forum.get_last_post()
         else:
+            super(Post, self).delete(*args, **kwargs)
             self.topic.update_post_count()
 
         self.topic.forum.update_post_count()
@@ -232,10 +249,11 @@ BAN_STATUS = (
 (1, _('Caution')),
 (2, _('Ban')))
 
+
 class Profile(models.Model):
-    user = AutoOneToOneField(User, related_name='pybb_profile', verbose_name=_('User'))
+    user = models.OneToOneField(User, related_name='pybb_profile', verbose_name=_('User'))
     signature = models.TextField(_('Signature'), blank=True, max_length=settings.PYBB_SIGNATURE_MAX_LENGTH)
-    signature_html = models.TextField(_('Signature HTML Version'), blank=True, max_length=settings.PYBB_SIGNATURE_MAX_LENGTH+30)
+    signature_html = models.TextField(_('Signature HTML Version'), blank=True, max_length=settings.PYBB_SIGNATURE_MAX_LENGTH + 30)
     time_zone = models.FloatField(_('Time zone'), choices=TZ_CHOICES, default=float(settings.PYBB_DEFAULT_TIME_ZONE))
     language = models.CharField(_('Language'), max_length=10, blank=True,
                                 choices=settings.LANGUAGES)
@@ -249,7 +267,6 @@ class Profile(models.Model):
         verbose_name = _('Profile')
         verbose_name_plural = _('Profiles')
 
-
     def save(self, *args, **kwargs):
         self.signature_html = mypostmarkup.markup(self.signature, auto_urls=False)
         super(Profile, self).save(*args, **kwargs)
@@ -265,7 +282,7 @@ class Profile(models.Model):
         return False
 
     def get_absolute_url(self):
-        return reverse('pybb_user', args=[self.user.username])
+        return reverse('pybb_user_details', args=[self.user.username])
 
 
 class Attachment(models.Model):
@@ -286,7 +303,7 @@ class Attachment(models.Model):
         return self.name
 
     def get_absolute_url(self):
-        return reverse('pybb_attachment', args=[self.hash])
+        return reverse('pybb_attachment_details', args=[self.hash])
 
     def size_display(self):
         size = self.size
@@ -297,7 +314,6 @@ class Attachment(models.Model):
         else:
             return '%.2fMb' % (size / float(1024 * 1024))
 
-
     def get_absolute_path(self):
         return os.path.join(settings.MEDIA_ROOT, settings.PYBB_ATTACHMENT_UPLOAD_TO,
                             self.path)
@@ -315,8 +331,8 @@ class ReadTracking(models.Model):
     `topic pk` --> `topic last post pk`
     """
 
-    user = AutoOneToOneField(User)
-    topics = JSONField(null=True)
+    user = models.OneToOneField(User)
+    topics = JSONField(null=True, blank=True)
     last_read = models.DateTimeField(null=True)
 
     class Meta:
