@@ -1,10 +1,5 @@
 from os.path import join
 
-
-PYBB_HEADER = 'PyBB'
-PYBB_TAGLINE = 'Yet another PyBB forum'
-PYBB_NOTICE = ''
-
 PYBB_TOPIC_PAGE_SIZE = 10
 PYBB_FORUM_PAGE_SIZE = 20
 PYBB_USERS_PAGE_SIZE = 20
@@ -26,7 +21,4 @@ PYBB_ATTACHMENT_ENABLE = True
 PYBB_SKIN = 'default'
 
 PYBB_ATTACHMENT_UPLOAD_TO = join('pybb_upload', 'attachments')
-PYBB_DEFAULT_AVATAR_URL = 'pybb/img/default_avatar.jpg'
-
-# Internal settings
-PYBB_DISABLE_NOTIFICATION = False
+PYBB_DEFAULT_AVATAR_URL = 'pybb/img/anonymous.gif'
